<?php
instant_vsl_use(['Controllers/Api/Page/ApiPageController', 'Models/Page']);

class InstantVSL_ApiAgPageController
{
    protected $page;

    protected $files;

    protected $page_fields = [
        'id', 'identifier', 'slug', 'theme_color', 'bg_color', 'title',
    ];

    protected $data_fields = [
        'theme_color', 'bg_color', 'title',
    ];

    public function index($request)
    {
        // get index
        $this->page = InstantVSL_Page::find($request->id, $this->page_fields);
        $page       = $this->page;

        $request->page = $page->slug;
        $request->slug = $page->slug;

        $data = new InstantVSL_ApiPageController;

        $content = $data->index($request, true);
        wp_send_json($content);
        exit;
    }

    public function create($request)
    {
        // append slug to request
        $this->page = InstantVSL_Page::find($request->id, $this->page_fields);

        $this->files = [
            ['name'   => 'index.html',
                'content' => $this->getIndexFileContent($request->content)],
            ['name'   => 'css/app.css',
                'content' => $this->getFileData('css/app.css')],
            ['name'   => 'css/chunk-vendors.css',
                'content' => $this->getFileData('css/chunk-vendors.css')],
            ['name'   => 'js/app.js',
                'content' => $this->getFileData('js/app.js')],
            ['name'   => 'js/chunk-vendors.js',
                'content' => $this->getFileData('js/chunk-vendors.js')],
            // ['name' => 'images/image.png',
            // 'from' => 'image.png']
        ];

        $filename = $request->name;
        $filename = sanitize_title($filename);
        $zipname  = $filename . '.zip';
        $zip      = new ZipArchive;
        $zip->open($zipname, ZipArchive::CREATE);

        $zip->addEmptyDir('css');
        $zip->addEmptyDir('js');

        if ($this->filesHave('images')) {
            $zip->addEmptyDir('images');
        }

        // wp_send_json($this->files); exit;

        foreach ($this->files as $file) {
            if (!empty($file['from'])) {
                $zip->addFile($file['from'], $file['name']);
            } else {
                $zip->addFromString($file['name'], $file['content']);
            }
        }

        // All files are added, so close the zip file.
        $zip->close();

        header('Content-Type: application/zip');
        header('Content-disposition: attachment; filename=' . $filename . '.zip');
        header('Content-Length: ' . filesize($zipname));
        readfile($zipname);

    }

    private function filesHave($type)
    {
        foreach ($this->files as $i) {
            if (strpos($i['name'], $type . '/') === 0) {
                return true;
            }
        }
        return false;
    }

    private function getFileData($file)
    {
        $path = plugin_dir_path(__FILE__) . '../../../../../public/live/' . $file;
        return file_get_contents($path);
    }

    private function getIndexFileContent($content)
    {
        foreach ($this->data_fields as $i) {
            if (!isset($this->page->{$i})) {
                $this->page->{$i} = '';
            }
        }
        return '<!DOCTYPE html><html lang="en"><head><meta charset="utf-8" /><meta http-equiv="X-UA-Compatible" content="IE=edge" /><meta name="viewport" content="width=device-width,initial-scale=1.0" /><title>' . stripslashes($this->page->title) . '</title>

  <!-- ADDITIONAL HEADER CONTENT HERE -->

  <!-- OPEN GRAPH OPTIONS -->
  <meta property="fb:app_id" content="" />
  <meta property="og:url" content="" />
  <meta property="og:title" content="Instant VSL" />
  <meta property="og:description" content="" />
  <meta property="og:type" content="website" />

  <style>
  /*  OPTIONAL CUSTOM STYLES  */
   body, html {
       background-color: #efefef;
   }
   </style>

<link rel="stylesheet" href="https://fonts.googleapis.com/css?family=Roboto:100,300,400,500,700,900" /><link rel="stylesheet" href="https://fonts.googleapis.com/css?family=Material+Icons" /><link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.8.2/css/all.css" /><link rel="stylesheet" type="text/css" href="css/chunk-vendors.css" /><link rel="stylesheet" type="text/css" href="css/app.css" /></head><body><div id="app" data-token="' . $this->page->identifier . '" data-theme_color="' . $this->page->theme_color . '" data-bg_color="' . $this->page->bg_color . '" data-hash="' . $content . '"></div><script src="js/chunk-vendors.js"></script><script src="js/app.js"></script>

<!-- ADDITIONAL BODY CONTENT -->

</body></html>';
    }

}
