<?php

instant_vsl_use([
  'Controllers/Api/ApiController',
  'Core/Time',
  'Models/Page',
  'Models/Visit',
  'Models/Click'
]);


class InstantVSL_ApiPageVisitsController extends InstantVSL_ApiController
{

  public function onReset($request)
  {
    $visits = new InstantVSL_Visit;
    $visits->wpDelete(['page_id' => $request->id]);

    $conversions = new InstantVSL_Click;
    $conversions->wpDelete(['page_id' => $request->id]);

    /// remove from page
    $page = InstantVSL_Page::where('id', $request->id)->first(['id']);
    if(!empty($page)) {
      $page->visits = 0;
      $page->visits_unique = 0;
      $page->save();
    }

      wp_send_json_success();
  }

  protected function getFirst($request)
  {
    return InstantVSL_Visit::where('page_id', $request->id)
                                    ->orderBy('created_ts', 'asc')
                                    ->first(['id','created_ts']);
  }

  protected function getVisits($page_id, $start_ts, $end_ts, $is_unique=null)
  {
  
    $q = InstantVSL_Visit::where('page_id', $page_id)
                                    // ->andWhereNull('block_id') fails
                                    ->andWhere('created_ts', '>', $start_ts)
                                    ->andWhere('created_ts', '<', $end_ts);
    if(!empty($is_unique)) {
      $q->andWhere('is_unique', 1);
    }

    // echo 'first visit: '.$q->count(); exit;
    return (int) $q->count();
  }

  protected function getConversions($page_id, $start_ts, $end_ts, $is_unique=null)
  {
    $q = InstantVSL_Visit::where('page_id', $page_id)
                                    ->andWhere('created_ts', '>', $start_ts)
                                    ->andWhere('created_ts', '<', $end_ts)
                                    ->andWhereNotNull('conversion_ts');
 
    return (int) $q->count();
  }



  public function index($request)
  {

    // if none, quit
    $valid = InstantVSL_Visit::where('page_id', $request->id)->first(['id']);

    if(empty($valid)) {
      // echo 'no visits'; exit;
      wp_send_json(['data' => 0, 'days' => 0, 'visits' => 0, 'visits_unique' => 0, 'conversions' => 0, 'conversions_unique' => 0]); exit;
    }
    // wp_send_json( $valid ); exit;

    $t = new InstantVSL_Time();


    if(empty($request->range)) {
      // 7 days
      $days = $t->getPastDays(7);
    }
    elseif(is_numeric($request->range)) {
      // x days
      $days = $t->getPastDays($request->range);
    }
    elseif($request->range == 'current_week') {
      $days = $t->getCurrentWeek();
    }
    elseif($request->range == 'current_month') {
      $days = $t->getCurrentMonth();
    }
    else { // == 'all'

      $first = $this->getFirst($request);

      // echo $first->created_ts / 86400; exit;

      $num = ceil(($t->ts - (int) $first->created_ts) / 86400);
      $num++; // add day

      $days = $t->getPastDays($num);

    }



    $count = 0;
    $total = 0;
    $total_unique = 0;

    $conversions = 0;
    $conversions_unique = 0;

    foreach($days as $i) {

      $days[$count]['visits'] = $this->getVisits($request->id, $i['start_ts'], $i['end_ts']);

      $total += $days[$count]['visits'];

      $days[$count]['visits_unique'] = $this->getVisits($request->id, $i['start_ts'], $i['end_ts'], 1);

      $total_unique += $days[$count]['visits_unique'];


      // conversions
      $days[$count]['conversions'] = $this->getConversions($request->id, $i['start_ts'], $i['end_ts']);
      $conversions += $days[$count]['conversions'];

      $days[$count]['conversions_unique'] = $this->getConversions($request->id, $i['start_ts'], $i['end_ts'], 1);
      $conversions_unique += $days[$count]['conversions_unique'];

      $count++;

    }

   
    $conversion_rate = ($conversions_unique / $total_unique) * 100;

    if(floor( $conversion_rate ) != $conversion_rate && $conversion_rate < 100) {
      $conversion_rate = number_format($conversion_rate, 1);
    }

    wp_send_json(['data' => $days, 'days' => count($days), 'visits' => $total, 'visits_unique' => $total_unique, 'conversions' => $conversions, 'conversions_unique' => $conversions_unique]);
  }
}
