<?php

function instant_vsl_decode_str($str)
{

    // main
    $str = urldecode(base64_decode($str));
    return $str;
}

function instant_vsl_parse_code($code)
{

    // echo "\n\n".'CODE: '."\n".$code."\n\n";

    // version 1.0.3
    if (strstr($code, '%') == false) {
        return instant_vsl_decode_str($code);
    }

    $code = urldecode($code);
    return stripslashes($code);
}

function instant_vsl_obj_atts($obj, $atts = [])
{

    if (empty($atts) || !is_array($atts)) {
        return $obj;
    }

    foreach ($atts as $key => $value) {
        if (isset($obj->{$key}) == false || is_null($obj->{$key}) || $obj->{$key} == '') {
            $obj->{$key} = $value;
        }
    }
    return $obj;
}

function instant_vsl_test_data($location, $file = '', $json = true, $print = true)
{
    $file = plugin_dir_path($location) . $file;

    if ($json) {
        $file .= '.json';
        $data = json_decode(file_get_contents($file));

        if ($print) {
            wp_send_json($data);exit;
        }
        return $data;
    }

    $data = file_get_contents($file);

    if ($print) {
        echo $data;exit;
    }

    return $data;

}

function instant_vsl_get_client_ip()
{
    if (!empty($_SERVER["HTTP_CLIENT_IP"])) {
        return $_SERVER["HTTP_CLIENT_IP"];
    } elseif (!empty($_SERVER["HTTP_X_FORWARDED_FOR"])) {
        return $_SERVER["HTTP_X_FORWARDED_FOR"];
    }
    return $_SERVER["REMOTE_ADDR"];
}

function instant_vsl_get_available_username($user)
{

    $username = strtolower(str_replace(' ', '', $user['first_name']));

    if (!username_exists($username)) {
        return $username;
    }

    if (!empty($user['last_name'])) {
        $username .= strtolower(str_replace(' ', '', $user['last_name']));
        if (!username_exists($username)) {
            return $username;
        }
    }

    $count = 2;
    while ($count < 1000) {
        if (!username_exists($username . $count)) {
            return $username . $count;
        }
        $count++;
    }

}

function instant_vsl_get_settings($getValue = '')
{
    global $instant_vsl_settings;
    global $instant_vsl_settings_set;

    if (empty($instant_vsl_settings_set)) {
        $instant_vsl_settings_set = 1;

        $settingsKey = InstantVSL::$plugin_app_id . '_settings';

        $instant_vsl_settings = get_option($settingsKey);
    }

    // get custom value

    if (!empty($getValue)) {
        if (!empty($instant_vsl_settings) && is_array($instant_vsl_settings) && !empty($instant_vsl_settings[$getValue])) {
            return $instant_vsl_settings[$getValue];
        }
        return '';
    }

    // return all
    return $instant_vsl_settings;

}

function instant_vsl_shortcode($classes, $mod = null)
{
    add_action('init', function () use ($classes, $mod) {

        if (!is_array($classes)) {
            $classes = [$classes];
        }

        foreach ($classes as $i) {
            if (!empty($mod)) {
                require (plugin_dir_path(__FILE__) . '../modules/' . $mod . '/app/Shortcodes/' . $i . '.php');
            } else {
                require (plugin_dir_path(__FILE__) . 'Shortcodes/' . $i . '.php');
            }

            $p  = explode('/', $i);
            $cl = InstantVSL::getPrefix() . array_pop($p) . 'Shortcode'; //.'PostType';

            new $cl();
        }

    });
}

function instant_vsl_post_type($classes, $mod = null)
{
    add_action('init', function () use ($classes, $mod) {

        if (!is_array($classes)) {
            $classes = [$classes];
        }

        foreach ($classes as $i) {
            if (!empty($mod)) {
                require (plugin_dir_path(__FILE__) . '../modules/' . $mod . '/app/PostTypes/' . $i . '.php');
            } else {
                require (plugin_dir_path(__FILE__) . 'PostTypes/' . $i . '.php');
            }

            $p  = explode('/', $i);
            $cl = InstantVSL::getPrefix() . array_pop($p); //.'PostType';

            new $cl();
        }

    });
}

// check if exists, and include if it does
function instant_vsl_mod_check($m)
{

    if (!is_array($m)) {
        $m = [$m];
    }

    foreach ($m as $mod) {

        $file = plugin_dir_path(__FILE__) . '../modules/' . $mod . '/' . $mod . '.php';
        if (file_exists($file)) {
            require $file;
        }

        $file = plugin_dir_path(__FILE__) . '../modules/' . $mod . '/bootstrap/app.php';

        if (file_exists($file)) {
            require $file;

        } else {
            $file = plugin_dir_path(__FILE__) . '../modules/' . $mod . '/bootstrap.php';
            if (file_exists($file)) {
                require $file;
            }
        }
    }

}

function instant_vsl_mod($m)
{
    if (!is_array($m)) {
        $m = [$m];
    }
    foreach ($m as $mod) {
        require plugin_dir_path(__FILE__) . '../modules/' . $mod . '/' . $mod . '.php';
    }
}

function instant_vsl_add_tag_atts($atts)
{

    if (empty($atts)) {
        return '';
    }

    if (!is_object($atts) && !is_array($atts)) {
        return ' ' . $atts;
    }

    $str = '';

    foreach ($atts as $key => $value) {
        $str .= ' ' . $key . '="' . $value . '"';
    }

    return $str;
}

function instant_vsl_embed_img($images, $args = [], $prependEach = '', $appendEach = '')
{
    if (!is_array($images)) {
        $images = [$images];
    }
    foreach ($images as $image) {
        echo $prependEach . '<img src="' . instant_vsl_img($image) . '"';
        if (!empty($args)) {
            foreach ($args as $key => $value) {
                echo ' ' . $key . '="' . $value . '"';
            }
        }
        echo ' />' . $appendEach;
    }
}

function instant_vsl_img($src)
{
    return instant_vsl_public_url('images/' . ltrim($src, ''));
}

/**
 * @param  string, array
 * @return [type]
 */
function instant_vsl_embed_js($scripts)
{
    if (!is_array($scripts)) {
        $scripts = [$scripts];
    }
    foreach ($scripts as $script) {
        echo instant_vsl_js($script, true);
    }
}

/**
 * @param  string, start with forward slash if public root, else will go to js folder
 * @param  boolean
 * @param  string
 * @return [type]
 */
function instant_vsl_js($src = 'app', $code = false, $atts = '')
{

    if (strstr($src, '//') == false) {

        if (strpos($src, '/') !== 0) {
            $src = 'js/' . $src;
        } else {
            $src = ltrim($src, '/');
        }

        $src = instant_vsl_public_url($src . '.js');

    }

    $info = instant_vsl_info();
    $src .= '?version=' . $info['version'];

    if ($code) {
        if (!empty($atts)) {

        }
        return '<script' . instant_vsl_add_tag_atts($atts) . ' src="' . $src . '"></script>';
    }
    return $src;
}

/**
 * @param  string, array
 * @return [type]
 */
function instant_vsl_embed_css($styles)
{
    if (!is_array($styles)) {
        $styles = [$styles];
    }
    foreach ($styles as $styles) {
        echo instant_vsl_css($styles, true);
    }
}

/**
 * @param  string, start with forward slash if public root, else will go to css folder
 * @param  boolean
 * @param  string
 * @return [type]
 */
function instant_vsl_css($href = 'app', $code = false, $atts = '')
{
    if (strstr($href, '//') == false) {

        if (strpos($href, '/') !== 0) {
            $href = 'css/' . $href;
        } else {
            $href = ltrim($href, '/');
        }

        $href = instant_vsl_public_url($href . '.css');
    }

    $info = instant_vsl_info();
    $href .= '?version=' . $info['version'];

    if ($code) {
        return '<link' . instant_vsl_add_tag_atts($atts) . ' rel="stylesheet" type="text/css" href="' . $href . '" />';
    }
    return $href;
}

function instant_vsl_public_url($file = '')
{
    return rtrim(plugins_url('', __FILE__), 'app') . 'public/' . $file;
}

function instant_vsl_metabox($classes, $mod = null)
{
    if (!is_array($classes)) {
        $classes = [$classes];
    }

    foreach ($classes as $i) {
        if (!empty($mod)) {
            require plugin_dir_path(__FILE__) . '../modules/' . $mod . '/app/MetaBoxes/' . $i . '.php';
        } else {
            require plugin_dir_path(__FILE__) . 'MetaBoxes/' . $i . '.php';
        }

        $p  = explode('/', $i);
        $cl = InstantVSL::getPrefix() . array_pop($p) . 'MetaBox';

        new $cl();

    }
}

function instant_vsl_config($key)
{
    global $instant_vsl_config_cache;

    if (isset($instant_vsl_config_cache[$key])) {
        return $instant_vsl_config_cache[$key];
    }

    $p    = explode('.', $key);
    $file = plugin_dir_path(__FILE__) . '../config/' . array_shift($p) . '.php';
    $re   = include $file;

    while ($i = array_shift($p)) {
        $re = $re[$i];
    }

    // save to cache
    $instant_vsl_config_cache[$key] = $re;

    return $re;

}

function instant_vsl_env($key, $default = '')
{
    global $instant_vsl_env_cache;

    if (isset($instant_vsl_env_cache)) {
        return isset($instant_vsl_env_cache[$key]) ? $instant_vsl_env_cache[$key] : $default;
    }

    $instant_vsl_env_cache = [];

    $path = ABSPATH . '../';

    $count = 0;

    while ($count < 10) {
        if (file_exists($path . '.env')) {
            $env = trim(file_get_contents($path . '.env'));
            break;
        }
        $path = '../' . $path;
        $count++; // extra protectedion
    }

    $rows = explode(PHP_EOL, $env);

    foreach ($rows as $row) {
        if (strstr($row, '=') == true) {
            $p                            = explode('=', $row);
            $instant_vsl_env_cache[$p[0]] = trim($p[1]);
        }
    }

    return isset($instant_vsl_env_cache[$key]) ? $instant_vsl_env_cache[$key] : $default;

}

function instant_vsl_use($classes, $mod = null)
{
    if (!is_array($classes)) {
        $classes = [$classes];
    }

    foreach ($classes as $i) {
        if (!empty($mod)) {

            include_once plugin_dir_path(__FILE__) . '../modules/' . $mod . '/app/' . $i . '.php';

        } else {
            include_once plugin_dir_path(__FILE__) . $i . '.php';
        }
    }

}

function instant_vsl_ini($classes, $mod = null, $customCl = null)
{

    if (!is_array($classes)) {
        $classes = [$classes];
    }

    foreach ($classes as $i) {
        if (!empty($mod)) {
            //echo plugin_dir_path(__FILE__).'../modules/'.$mod.'/app/'.$i.'.php'; exit;
            require plugin_dir_path(__FILE__) . '../modules/' . $mod . '/app/' . $i . '.php';
        } else {
            require plugin_dir_path(__FILE__) . $i . '.php';
        }

        if (!empty($customCl)) {
            $cl = InstantVSL::getPrefix() . $customCl;

        } else {
            $p  = explode('/', $i);
            $cl = InstantVSL::getPrefix() . array_pop($p);
        }

        new $cl();

    }
}

function instant_vsl_que_actions($actions, $mod = null)
{
    foreach ($actions as $action) {

        add_action($action, function () use ($action, $mod) {

            if (!empty($mod)) {
                $file = plugin_dir_path(__FILE__) . '../modules/' . $mod . '/bootstrap/actions/' . str_replace('_', '-', $action) . '.php';
            } else {
                $file = plugin_dir_path(__FILE__) . '../bootstrap/actions/' . str_replace('_', '-', $action) . '.php';
            }

            $list = include ($file);

            instant_vsl_ini($list, $mod);

        });
    }
}

function instant_vsl_info($path = null, $item = null, $includeContent = false, $includeRows = null)
{

    if (empty($path)) {
        $path = plugin_dir_path(InstantVSL::$plugin_app_file) . InstantVSL::$plugin_app_cl . '.php';
        // echo 'get this plugin file path<br>'.$path; exit;
    } elseif (!empty($path) && strstr($path, '/') == false) {
        $path = plugin_dir_path(InstantVSL::$plugin_app_file) . '../' . $path . '/' . $path . '.php';
        // echo 'get other plugin '.$path; exit;
    }

    if (!file_exists($path)) {
        return null;
    }

    $content = file_get_contents($path);

    $rows = explode(PHP_EOL, file_get_contents($path));

    $info = [];

    foreach ($rows as $row) {

        if (strstr($row, '=') == true) {

            $p = explode('=', $row);

            $value = explode("'", $p[1]);

            if (empty($value[1])) {
                continue;
            }

            $id = explode('$', $p[0]);
            $id = trim(array_pop($id));

            $info[$id] = $value[1];

        }
        // plugin details
        elseif (strstr($row, ':') == true) {
            $p   = explode(':', $row);
            $id  = array_shift($p);
            $val = implode(':', $p);

            $id        = str_replace(' ', '_', strtolower(trim($id)));
            $info[$id] = trim($val);
        } elseif (strpos(trim($row), 'class ') === 0) {
            $p                    = explode(' ', trim($row));
            $info['master_class'] = $p[1];
        }

    }

    if (!empty($includeContent)) {
        return [
            'content' => $content,
            'data'    => $info,
        ];
    }

    if (!empty($item)) {
        return $info[$item];
    }

    return $info;

}
